Package: adegenet
Title: Exploratory Analysis of Genetic and Genomic Data
Version: 2.1.11
Authors@R: 
    c(person(given = "Thibaut",
             family = "Jombart",
             role = "aut",
             comment = c(ORCID = "0000-0003-2226-8692")),
      person(given = "Zhian N.",
             family = "Kamvar",
             role = c("aut", "cre"),
             email = "zkamvar@gmail.com",
             comment = c(ORCID = "0000-0003-1458-7108")),
      person(given = "Caitlin",
             family = "Collins",
             role = "ctb"),
      person(given = "Roman",
             family = "Lustrik",
             role = "ctb"),
      person(given = "Marie-Pauline",
             family = "Beugin",
             role = "ctb"),
      person(given = "Brian J.",
             family = "Knaus",
             role = "ctb"),
      person(given = "Peter",
             family = "Solymos",
             role = "ctb"),
      person(given = "Vladimir",
             family = "Mikryukov",
             role = "ctb"),
      person(given = "Klaus",
             family = "Schliep",
             role = "ctb"),
      person(given = "Tiago",
             family = "Maié",
             role = "ctb"),
      person(given = "Libor",
             family = "Morkovsky",
             role = "ctb"),
      person(given = "Ismail",
             family = "Ahmed",
             role = "ctb"),
      person(given = "Anne",
             family = "Cori",
             role = "ctb"),
      person(given = "Federico",
             family = "Calboli",
             role = "ctb"),
      person(given = "RJ",
             family = "Ewing",
             role = "ctb"),
      person(given = "Frédéric",
             family = "Michaud",
             role = "ctb"),
      person(given = "Rebecca",
             family = "DeCamp",
             role = "ctb"),
      person(given = "Alexandre",
             family = "Courtiol",
             role = "ctb",
             comment = c(ORCID = "0000-0003-0637-2959")),
      person(given = "Lindsay V.",
             family = "Clark",
             role = "ctb",
             comment = c(ORCID = "0000-0002-3881-9252")),
       person(given = "Pavel",
              family = "Dimens",
              role = "ctb",
              comment = c(ORCID = "0000-0003-3823-0373")),
       person(given = "Max",
              family = "Coulter",
              role = "ctb"),
       person(given = "Ivan",
              family = "Krylov",
              role = "ctb")
       )
Description: Toolset for the exploration of genetic and genomic
    data. Adegenet provides formal (S4) classes for storing and handling
    various genetic data, including genetic markers with varying ploidy
    and hierarchical population structure ('genind' class), alleles counts
    by populations ('genpop'), and genome-wide SNP data ('genlight'). It
    also implements original multivariate methods (DAPC, sPCA), graphics,
    statistical tests, simulation tools, distance and similarity measures,
    and several spatial methods. A range of both empirical and simulated
    datasets is also provided to illustrate various methods.
License: GPL (>= 2)
URL: https://github.com/thibautjombart/adegenet
Depends: R (>= 2.14), methods, ade4
Imports: utils, stats, grDevices, MASS, igraph, ape, shiny, ggplot2,
        seqinr, parallel, boot, reshape2, dplyr (>= 0.4.1), vegan
Suggests: adespatial, pegas, hierfstat, maps, spdep, interp, splancs,
        poppr, testthat
Encoding: UTF-8
LazyLoad: yes
RoxygenNote: 7.3.2
Collate: 'adegenet.package.R' 'datasets.R' 'dist.genlight.R'
        'orthobasis.R' 'classes.R' 'constructors.R' 'accessors.R'
        'basicMethods.R' 'handling.R' 'auxil.R' 'minorAllele.R'
        'setAs.R' 'SNPbin.R' 'strataMethods.R' 'hierarchyMethods.R'
        'glHandle.R' 'glFunctions.R' 'glSim.R' 'find.clust.R'
        'hybridize.R' 'scale.R' 'fstat.R' 'import.R' 'seqTrack.R'
        'chooseCN.R' 'genind2genpop.R' 'loadingplot.R' 'sequences.R'
        'gstat.randtest.R' 'makefreq.R' 'colorplot.R' 'monmonier.R'
        'spca.R' 'coords.monmonier.R' 'haploGen.R' 'old2new.R'
        'global_local_tests.R' 'dapc.R' 'compoplot.R' 'xvalDapc.R'
        'haploPop.R' 'PCtest.R' 'dist.genpop.R' 'Hs.R' 'propShared.R'
        'export.R' 'HWE.R' 'propTyped.R' 'inbreeding.R' 'glPlot.R'
        'gengraph.R' 'simOutbreak.R' 'mutations.R' 'snpposi.R'
        'snpzip.R' 'pairDist.R' 'snapclust.R' 'AIC.snapclust.R'
        'AICc.snapclust.R' 'BIC.snapclust.R' 'KIC.snapclust.R'
        'snapclust.choose.k.R' 'servers.R' 'showmekittens.R'
        'spca_randtest.R' 'export_to_mvmapper.R' 'doc_C_routines.R'
        'zzz.R'
NeedsCompilation: yes
Packaged: 2025-02-06 21:23:20 UTC; zkamvar
Author: Thibaut Jombart [aut] (<https://orcid.org/0000-0003-2226-8692>),
  Zhian N. Kamvar [aut, cre] (<https://orcid.org/0000-0003-1458-7108>),
  Caitlin Collins [ctb],
  Roman Lustrik [ctb],
  Marie-Pauline Beugin [ctb],
  Brian J. Knaus [ctb],
  Peter Solymos [ctb],
  Vladimir Mikryukov [ctb],
  Klaus Schliep [ctb],
  Tiago Maié [ctb],
  Libor Morkovsky [ctb],
  Ismail Ahmed [ctb],
  Anne Cori [ctb],
  Federico Calboli [ctb],
  RJ Ewing [ctb],
  Frédéric Michaud [ctb],
  Rebecca DeCamp [ctb],
  Alexandre Courtiol [ctb] (<https://orcid.org/0000-0003-0637-2959>),
  Lindsay V. Clark [ctb] (<https://orcid.org/0000-0002-3881-9252>),
  Pavel Dimens [ctb] (<https://orcid.org/0000-0003-3823-0373>),
  Max Coulter [ctb],
  Ivan Krylov [ctb]
Maintainer: Zhian N. Kamvar <zkamvar@gmail.com>
Repository: CRAN
Date/Publication: 2025-02-06 22:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 11:16:02 UTC; unix
Archs: adegenet.so.dSYM
