# Generated by roxygen2: do not edit by hand

S3method("[",haploGen)
S3method(AIC,snapclust)
S3method(AICc,snapclust)
S3method(BIC,snapclust)
S3method(KIC,snapclust)
S3method(as.POSIXct,haploGen)
S3method(as.data.frame,genind)
S3method(as.data.frame,genlight)
S3method(as.data.frame,genpop)
S3method(as.genpop,genind)
S3method(as.igraph,haploGen)
S3method(as.integer,SNPbin)
S3method(as.lda,dapc)
S3method(as.list,genlight)
S3method(as.matrix,genind)
S3method(as.matrix,genlight)
S3method(as.matrix,genpop)
S3method(c,SNPbin)
S3method(cbind,SNPbin)
S3method(cbind,genlight)
S3method(colorplot,default)
S3method(colorplot,spca)
S3method(compoplot,dapc)
S3method(compoplot,matrix)
S3method(compoplot,snapclust)
S3method(dapc,data.frame)
S3method(dapc,dudi)
S3method(dapc,genind)
S3method(dapc,genlight)
S3method(dapc,matrix)
S3method(export_to_mvmapper,dapc)
S3method(export_to_mvmapper,default)
S3method(export_to_mvmapper,dudi)
S3method(export_to_mvmapper,spca)
S3method(find.clusters,data.frame)
S3method(find.clusters,genind)
S3method(find.clusters,genlight)
S3method(find.clusters,matrix)
S3method(findMutations,DNAbin)
S3method(gengraph,DNAbin)
S3method(gengraph,default)
S3method(gengraph,dist)
S3method(gengraph,genind)
S3method(gengraph,genpop)
S3method(gengraph,matrix)
S3method(get.likelihood,seqTrack)
S3method(graphMutations,DNAbin)
S3method(labels,haploGen)
S3method(loadingplot,default)
S3method(loadingplot,glPca)
S3method(pairDistPlot,DNAbin)
S3method(pairDistPlot,default)
S3method(pairDistPlot,dist)
S3method(pairDistPlot,genind)
S3method(pairDistPlot,matrix)
S3method(plot,haploGen)
S3method(plot,monmonier)
S3method(plot,seqTrack)
S3method(plot,spca)
S3method(predict,dapc)
S3method(print,dapc)
S3method(print,genindSummary)
S3method(print,genpopSummary)
S3method(print,glPca)
S3method(print,haploGen)
S3method(print,monmonier)
S3method(print,spca)
S3method(rbind,genlight)
S3method(scatter,dapc)
S3method(scatter,glPca)
S3method(screeplot,spca)
S3method(seqTrack,default)
S3method(seqTrack,haploGen)
S3method(seqTrack,matrix)
S3method(snpposi.plot,DNAbin)
S3method(snpposi.plot,integer)
S3method(snpposi.plot,numeric)
S3method(snpposi.test,DNAbin)
S3method(snpposi.test,integer)
S3method(snpposi.test,numeric)
S3method(spca,data.frame)
S3method(spca,default)
S3method(spca,genind)
S3method(spca,genpop)
S3method(spca,matrix)
S3method(summary,dapc)
S3method(summary,spca)
S3method(xvalDapc,data.frame)
S3method(xvalDapc,default)
S3method(xvalDapc,genind)
S3method(xvalDapc,genlight)
S3method(xvalDapc,matrix)
export("addStrata<-")
export("hier<-")
export("nameStrata<-")
export("setPop<-")
export("splitStrata<-")
export("strata<-")
export(.genlab)
export(.internal_C_routines)
export(.readExt)
export(.render.server.info)
export(.rmspaces)
export(AICc)
export(Hs)
export(Hs.test)
export(KIC)
export(addStrata)
export(adegenetIssues)
export(adegenetTutorial)
export(adegenetWeb)
export(as.genind)
export(as.genpop)
export(chooseCN)
export(compoplot)
export(df2genind)
export(export_to_mvmapper)
export(extract.PLINKmap)
export(gengraph)
export(genind)
export(genind2df)
export(genind2genpop)
export(genpop)
export(hier)
export(hybridize)
export(is.genind)
export(is.genpop)
export(loadingplot)
export(makefreq)
export(minorAllele)
export(nameStrata)
export(old2new_genind)
export(old2new_genlight)
export(old2new_genpop)
export(read.PLINK)
export(read.fstat)
export(read.genepop)
export(read.genetix)
export(read.snp)
export(read.structure)
export(repool)
export(sample.haploGen)
export(scaleGen)
export(seqTrack)
export(setPop)
export(showmekittens)
export(snapclust)
export(snapclust.choose.k)
export(snpposi.plot)
export(snpposi.test)
export(spca_randtest)
export(splitStrata)
export(strata)
export(tab)
export(xvalDapc)
exportMethods(initialize)
exportMethods(makefreq)
exportMethods(scaleGen)
exportPattern("^[^\\.]")
import(ade4)
import(grDevices)
import(graphics)
import(methods)
import(parallel)
import(stats)
import(utils)
importFrom(MASS,"lda")
importFrom(ape,"[.DNAbin")
importFrom(ape,"as.DNAbin")
importFrom(ape,"as.DNAbin.alignment")
importFrom(ape,"as.DNAbin.character")
importFrom(ape,"as.DNAbin.list")
importFrom(ape,"as.character.DNAbin")
importFrom(ape,"as.list.DNAbin")
importFrom(ape,"as.matrix.DNAbin")
importFrom(ape,"c.DNAbin")
importFrom(ape,"cbind.DNAbin")
importFrom(ape,"dist.dna")
importFrom(ape,"labels.DNAbin")
importFrom(ape,"print.DNAbin")
importFrom(ape,"rbind.DNAbin")
importFrom(ape,"seg.sites")
importFrom(dplyr,bind_rows)
importFrom(ggplot2,"aes")
importFrom(ggplot2,"coord_flip")
importFrom(ggplot2,"geom_boxplot")
importFrom(ggplot2,"geom_density")
importFrom(ggplot2,"geom_jitter")
importFrom(ggplot2,"geom_rug")
importFrom(ggplot2,"geom_violin")
importFrom(ggplot2,"ggplot")
importFrom(ggplot2,"guide_legend")
importFrom(ggplot2,"guides")
importFrom(ggplot2,"labs")
importFrom(ggplot2,"xlim")
importFrom(igraph,"E")
importFrom(igraph,"E<-")
importFrom(igraph,"V")
importFrom(igraph,"V<-")
importFrom(igraph,"as.igraph")
importFrom(igraph,"clusters")
importFrom(igraph,"graph.adjacency")
importFrom(igraph,"graph.data.frame")
importFrom(igraph,"layout.fruchterman.reingold")
importFrom(igraph,"plot.igraph")
importFrom(igraph,"print.igraph")
importFrom(reshape2,colsplit)
importFrom(seqinr,s2c)
importFrom(shiny,"renderPrint")
importFrom(shiny,"runApp")
importFrom(vegan,orditorp)
useDynLib(adegenet, .registration = TRUE)
