<!-- badges: start -->
[![R-CMD-check](https://github.com/thibautjombart/adegenet/workflows/R-CMD-check/badge.svg)](https://github.com/thibautjombart/adegenet/actions)

[![CRAN Status Badge](https://www.r-pkg.org/badges/version/adegenet)](https://cran.r-project.org/package=adegenet)

[![CRAN Downloads](https://cranlogs.r-pkg.org/badges/adegenet)](https://cran.r-project.org/package=adegenet)

[![Downloads from Rstudio mirror](https://cranlogs.r-pkg.org/badges/grand-total/adegenet)](https://cran.r-project.org/package=adegenet)

  <!-- badges: end -->

adegenet
========
*adegenet*: a R Package for the Multivariate Analysis of Genetic Markers

This wiki is dedicated to the development of *adegenet*.
It is relevant for developers of the package, developers of other packages depending on *adegenet*, and for users who want to be using the latest features as well.

The *adegenet website* is available at http://adegenet.r-forge.r-project.org/.

The following sections are available:
- [Installing adegenet devel](https://github.com/thibautjombart/adegenet/wiki/Installing-adegenet-devel)
- [Tutorials](https://github.com/thibautjombart/adegenet/wiki/Tutorials)
- [Ask a question](https://github.com/thibautjombart/adegenet/wiki/Questions)
- [Features request / bug report](https://github.com/thibautjombart/adegenet/issues)
- [Branches](https://github.com/thibautjombart/adegenet/wiki/Branches)
- [Contributors](https://github.com/thibautjombart/adegenet/wiki/Contributors)
