#' Internal C routines
#'
#' These functions are internal C routines used in adegenet. Do not use them
#' unless you know what you are doing.
#'
#' @aliases CheckAllSeg GLdotProd GLsumFreq GLsumInt binIntToBytes bytesToBinInt
#' bytesToInt nb_shared_all
#'
#' @author Thibaut Jombart
#'
#' @export
#'
#' @rdname doc_C_routines
#' 
".internal_C_routines" <- NULL
